# Orca
#
# Copyright 2004-2009 Sun Microsystems Inc.
# Copyright 2026 Igalia, S.L.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., Franklin Street, Fifth Floor,
# Boston MA  02110-1301 USA.

# This file is auto-generated by tools/generate_braille_table_names.py.
# Do not edit manually. Re-run the tool after updating liblouis.

# pylint: disable=too-many-lines

"""Braille translation table names for localization.

These are in their own module so that we can present them in the correct
language when users change the synthesizer language on the fly without
having to reload a bunch of modules."""

from .orca_i18n import _  # pylint: disable=import-error

TABLE_NAMES: dict[str, str] = {
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "afr-za-g1.ctb": _("Afrikaans, uncontracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "afr-za-g2.ctb": _("Afrikaans, contracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "akk-borger.utb": _("Akkadian, Borger"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "akk.utb": _("Akkadian, U.S."),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ancient-languages-borger.utb": _("Ancient languages, Borger"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ancient-languages-us.utb": _("Ancient languages, U.S."),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ar-ar-comp8.utb": _("Arabic, computer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ar-ar-g1.utb": _("Arabic, uncontracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ar-ar-g2.ctb": _("Arabic, contracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "as-in-g1.utb": _("Assamese"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "aw-in-g1.utb": _("Awadhi"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ba.utb": _("Bashkir"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "be-in-g1.utb": _("Bengali"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "bel-comp.utb": _("Belarusian, computer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "bel-detailed.utb": _("Belarusian, with capitals"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "bel.utb": _("Belarusian"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "bg.ctb": _("Bulgarian, computer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "bg.utb": _("Bulgarian"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "bh.ctb": _("Bihari"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "bo.ctb": _("Tibetan, computer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "boxes.ctb": _("boxes"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "br-in-g1.utb": _("Braj"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ca-g1.ctb": _("Catalan"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "chr-us-g1.ctb": _("Cherokee"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ckb-g1.ctb": _("Kurdish"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "cop-eg-comp8.utb": _("Coptic, computer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "cop.utb": _("Coptic"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "cs-comp8.utb": _("Czech, computer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "cs-g1.ctb": _("Czech"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "cuneiform-transliterated-compact.utb": _("Cuneiform, transliterated, compact"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "cuneiform-transliterated.utb": _("Cuneiform, transliterated"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "cy-cy-g1.utb": _("Welsh, uncontracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "cy-cy-g2.ctb": _("Welsh, contracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "da-dk-g08.ctb": _("Danish, computer, 2022"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "da-dk-g08_1993.ctb": _("Danish, computer, 1993"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "da-dk-g16-lit_1993.ctb": _("Danish, uncontracted, 6-dot, 1993"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "da-dk-g16.ctb": _("Danish, uncontracted, 6-dot, 2022"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "da-dk-g16_1993.ctb": _("Danish, uncontracted, 6-dot, 1993"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "da-dk-g18.ctb": _("Danish, uncontracted, 8-dot, 2022"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "da-dk-g18_1993.ctb": _("Danish, uncontracted, 8-dot, 1993"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "da-dk-g26-lit_1993.ctb": _("Danish, contracted, 6-dot, 1993"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "da-dk-g26.ctb": _("Danish, contracted, 6-dot, 2022"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "da-dk-g26_1993.ctb": _("Danish, contracted, 6-dot, 1993"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "da-dk-g26l-lit_1993.ctb": _("Danish, partially contracted, 6-dot, 1993"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "da-dk-g26l_1993.ctb": _("Danish, partially contracted, 6-dot, 1993"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "da-dk-g28.ctb": _("Danish, contracted, 8-dot, 2022"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "da-dk-g28_1993.ctb": _("Danish, contracted, 8-dot, 1993"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "da-dk-g28l_1993.ctb": _("Danish, partially contracted, 8-dot, 1993"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "de-chess.ctb": _("de-chess"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "de-comp6.utb": _("German, computer, 6-dot"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "de-de-comp8.ctb": _("German, computer, 8-dot"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "de-g0-detailed.utb": _("German, uncontracted, with capitals"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "de-g0.utb": _("German, uncontracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "de-g1-detailed.ctb": _("German, partially contracted, with capitals"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "de-g1.ctb": _("German, partially contracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "de-g2-detailed.ctb": _("German, contracted, with capitals"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "de-g2.ctb": _("German, contracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "dra.ctb": _("Dravidian, computer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "el.ctb": _("Greek"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "en-chess.ctb": _("en-chess"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "en-g3.ctb": _("English, contracted, grade 3"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "en-gb-comp8.ctb": _("English, U.K., computer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "en-gb-g1.utb": _("English, U.K., uncontracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "en-GB-g2.ctb": _("English, U.K., contracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "en-in-g1.ctb": _("en-in-g1"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "en-nabcc.utb": _("English, U.S., computer, NABCC"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "en-ueb-g1.ctb": _("English, unified, uncontracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "en-ueb-g2.ctb": _("English, unified, contracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "en-ueb-math.ctb": _("en-ueb-math"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "en-us-comp6.ctb": _("English, U.S., computer, 6-dot"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "en-us-comp8-ext.utb": _("English, U.S., computer, 8-dot"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "en-us-comp8.ctb": _("en-us-comp8"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "en-us-g1.ctb": _("English, U.S., uncontracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "en-us-g2.ctb": _("English, U.S., contracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "en-us-interline.ctb": _("en-us-interline"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "en-us-mathtext.ctb": _("en-us-mathtext"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "en_CA.ctb": _("English, Canada, computer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "eo-g1-x-system.ctb": _("Esperanto x-system"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "eo-g1.ctb": _("Esperanto"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "Es-Es-G0.utb": _("Spanish, computer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "es-g1.ctb": _("Spanish, uncontracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "es-g2.ctb": _("Spanish, contracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "es-no.utb": _("Spanish, Norway"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "et-6dot.utb": _("Estonian"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "et-g0.utb": _("Estonian, computer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ethio-g1.ctb": _("Ethiopic"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "fa-ir-comp8.ctb": _("Persian, computer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "fa-ir-g1.utb": _("Persian"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "fi-fi-8dot.ctb": _("Finnish, computer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "fi.utb": _("Finnish"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "fil-g2.ctb": _("Filipino, contracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "fr-bfu-comp6.utb": _("French, uncontracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "fr-bfu-comp8.utb": _("French, computer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "fr-bfu-g2.ctb": _("French, contracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ga-g1.utb": _("Irish, uncontracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ga-g2.ctb": _("Irish, contracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "gd.ctb": _("Scottish Gaelic, computer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "gon.ctb": _("Gondi"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "grc-international-en-composed.utb": _("Greek, international, English, composed"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "grc-international-en.utb": _("Greek, international, English, decomposed"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "grc-international-es.utb": _("Greek, international, Spanish"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "gu-in-g1.utb": _("Gujarati"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "haw-us-g1.ctb": _("Hawaiian"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "hbo-cantillated.utb": _("Hebrew, classical, cantillated"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "hbo-slim.utb": _("Hebrew, classical, slim"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "hbo.utb": _("Hebrew, classical, IHBC"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "he-IL-comp8.utb": _("Hebrew, modern, computer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "he-IL.utb": _("Hebrew, modern"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "hi-in-g1.utb": _("Hindi"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "hr-comp8.utb": _("Croatian, computer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "hr-g1.ctb": _("Croatian"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "hu-hu-comp8.ctb": _("Hungarian, computer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "hu-hu-g1.ctb": _("Hungarian, uncontracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "hu-hu-g2.ctb": _("Hungarian, contracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "hy.ctb": _("Armenian, computer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "IPA.utb": _("International Phonetic Alphabet"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "is.ctb": _("is"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "it-it-comp6.utb": _("Italian"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "it-it-comp8.utb": _("Italian, computer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "iu-ca-g1.ctb": _("Inuktitut"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ja-kantenji-ucs2.utb": _("Japanese, Kantenji"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ja-kantenji.utb": _("Japanese, Kantenji"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ja-rokutenkanji.utb": _("Japanese, Rokuten Kanji"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ka-in-g1.utb": _("Kannada"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ka.utb": _("Georgian"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "kh-in-g1.utb": _("Khasi"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "kk.utb": _("Kazakh"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "km-g1.utb": _("Khmer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ko-2006-g1.ctb": _("Korean, uncontracted, 2006"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ko-2006-g2.ctb": _("Korean, contracted, 2006"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ko-g1.ctb": _("Korean, uncontracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ko-g2.ctb": _("Korean, contracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "kok.ctb": _("Konkani"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "kru.ctb": _("Kurukh"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ks-in-g1.utb": _("ks-in-g1"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "lg-ug-g1.utb": _("Luganda"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "lo-g1.utb": _("Lao"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "lt-6dot.utb": _("Lithuanian, 6-dot"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "lt-8dot.utb": _("Lithuanian, 8-dot"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "lt.ctb": _("lt"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "Lv-Lv-g1.utb": _("Latvian"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "mao-nz-g1.ctb": _("Maori"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "mk-g1.utb": _("Macedonian"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ml-in-g1.utb": _("Malayalam"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "mn-in-g1.utb": _("Manipuri"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "mn-MN-g1.utb": _("Mongolian, uncontracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "mn-MN-g2.ctb": _("Mongolian, contracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "mr-in-g1.utb": _("Marathi"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ms-my-g2.ctb": _("Malay"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "mt.ctb": _("Maltese, computer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "mun.ctb": _("Munda"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "mwr.ctb": _("Marwari"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "my-g1.utb": _("Burmese, uncontracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "my-g2.ctb": _("Burmese, contracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ne.ctb": _("ne"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "nl-comp8.utb": _("Dutch, computer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "no-no-8dot-fallback-6dot-g0.utb": _("Norwegian, uncontracted, 8-dot, 6-dot fallback"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "no-no-8dot.utb": _("Norwegian, uncontracted, 8-dot"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "no-no-comp8.ctb": _("Norwegian, computer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "no-no-g0.utb": _("Norwegian, uncontracted, 6-dot"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "no-no-g1.ctb": _("Norwegian, contracted, grade 1"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "no-no-g2.ctb": _("Norwegian, contracted, grade 2"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "no-no-g3.ctb": _("Norwegian, contracted, grade 3"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "no-no-generic.ctb": _("no-no-generic"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "np-in-g1.utb": _("Nepali"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "nso-za-g1.utb": _("Sepedi, uncontracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "nso-za-g2.ctb": _("Sepedi, contracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ny-mw.utb": _("Chichewa"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "or-in-g1.utb": _("Oriya"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ovd-6g0.utb": _("Elfdalian, 6-dot"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ovd-8g0.utb": _("Elfdalian, 8-dot"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "pi.ctb": _("Pali"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "pl-pl-comp8.ctb": _("Polish, computer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "Pl-Pl-g1.utb": _("Polish"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "pt-comp6.utb": _("Portuguese, computer, 6-dot"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "pt-pt-comp8.ctb": _("Portuguese, computer, 8-dot"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "pt-pt-g1.utb": _("Portuguese, uncontracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "pt-pt-g2.ctb": _("Portuguese, contracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "pu-in-g1.utb": _("Punjabi"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ro-g0.utb": _("Romanian, uncontracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ro.ctb": _("Romanian, computer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ru-comp6.utb": _("Russian, computer, 6-dot"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ru-comp8.utb": _("Russian, computer, 8-dot"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ru-litbrl-detailed.utb": _("Russian, with capitals"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ru-litbrl.ctb": _("Russian"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ru-ru-g1.ctb": _("Russian, contracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "rw-rw-g1.utb": _("Kinyarwanda"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sa-in-g1.utb": _("Sanskrit"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sah.utb": _("Yakut"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "se-se.ctb": _("se-se"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "si-in-g1.utb": _("Sindhi"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sin.utb": _("sin"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sk-g1.ctb": _("Slovak"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sk-sk-g1.utb": _("sk-sk-g1"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sk-sk.utb": _("sk-sk"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sl-si-comp8.ctb": _("Slovenian, computer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sl-si-g1.utb": _("Slovenian"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "smi-6g0.utb": _("Sami, 6-dot"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "smi-8g0.utb": _("Sami, 8-dot"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sot-za-g1.ctb": _("Sesotho, uncontracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sot-za-g2.ctb": _("Sesotho, contracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sr-Cyrl.ctb": _("Serbian, Cyrillic"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sr-g1.ctb": _("Serbian"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sv-1989.ctb": _("Swedish, computer, 1989"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sv-1996.ctb": _("Swedish, computer, 1996"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sv-6g0d.utb": _("Swedish, uncontracted, 6-dot, detailed"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sv-6g0p.utb": _("Swedish, uncontracted, 6-dot, phonetics"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sv-6g1d.ctb": _("Swedish, partially contracted, 6-dot, detailed"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sv-6g2d.ctb": _("Swedish, contracted, 6-dot, detailed"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sv-8g0d.utb": _("Swedish, uncontracted, 8-dot, detailed"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sv-8g0n.utb": _("Swedish, uncontracted, 8-dot"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sv-8g1d.ctb": _("Swedish, partially contracted, 8-dot, detailed"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sv-8g1n.ctb": _("Swedish, partially contracted, 8-dot"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sv-8g2d.ctb": _("Swedish, contracted, 8-dot, detailed"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sv-8g2n.ctb": _("Swedish, contracted, 8-dot"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sv-g0.utb": _("Swedish, uncontracted, 6-dot"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sv-g1.ctb": _("Swedish, partially contracted, 6-dot"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sv-g2.ctb": _("Swedish, contracted, 6-dot"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sw-ke-g1-2.ctb": _("Swahili, contracted, grade 1.2"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sw-ke-g1-3.ctb": _("Swahili, contracted, grade 1.3"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sw-ke-g1-4.ctb": _("Swahili, contracted, grade 1.4"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sw-ke-g1-5.ctb": _("Swahili, contracted, grade 1.5"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sw-ke-g1.utb": _("Swahili, uncontracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sw-ke-g2.ctb": _("Swahili, contracted, grade 2"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "syc.utb": _("Syriac"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ta-ta-g1.ctb": _("Tamil"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ta.ctb": _("Tamil, computer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "te-in-g1.utb": _("Telugu"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "th-comp8-backward.utb": _("Thai, computer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "th-g0.utb": _("Thai, uncontracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "th-g1.utb": _("Thai, partially contracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "th-g2.ctb": _("Thai, contracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "tr-g1.ctb": _("Turkish, uncontracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "tr-g2.ctb": _("Turkish, contracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "tr.ctb": _("Turkish, computer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "tsn-za-g1.ctb": _("Setswana, uncontracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "tsn-za-g2.ctb": _("Setswana, contracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "tt.utb": _("Tatar"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "uga.utb": _("Ugaritic"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "uk-comp.utb": _("Ukrainian, computer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "uk-detailed.utb": _("Ukrainian, with capitals"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "uk.utb": _("Ukrainian"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "unicode-braille.utb": _("unicode-braille"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ur-pk-g1.utb": _("Urdu, uncontracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ur-pk-g2.ctb": _("Urdu, contracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "uz-g1.utb": _("Uzbek"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ve-za-g1.utb": _("Tshivenda, uncontracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ve-za-g2.ctb": _("Tshivenda, contracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "vi-cb8.utb": _("Vietnamese, computer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "vi-saigon-g1.ctb": _("Vietnamese, Saigon, contracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "vi-vn-g0.utb": _("Vietnamese, uncontracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "vi-vn-g1.ctb": _("Vietnamese, partially contracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "vi-vn-g2.ctb": _("Vietnamese, contracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "xh-za-g1.utb": _("isiXhosa, uncontracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "xh-za-g2.ctb": _("isiXhosa, contracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "yi.utb": _("Yiddish"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "zh-chn.ctb": _("Mandarin, mainland China, without tones"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "zh-hk.ctb": _("Cantonese"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "zh-tw.ctb": _("Mandarin, Taiwan, bopomofo"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "zhcn-cbs.ctb": _("Mandarin, mainland China, common braille scheme"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "zhcn-g1.ctb": _("Mandarin, mainland China, with tones"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "zhcn-g2.ctb": _("Mandarin, mainland China, double-phonic"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "zu-za-g1.utb": _("isiZulu, uncontracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "zu-za-g2.ctb": _("isiZulu, contracted"),
}
