"""Orca"s application scripts."""

__all__ = [
    "Thunderbird",
    "evolution",
    "gajim",
    "gnome-shell",
    "kwin",
    "notification-daemon",
    "pidgin",
    "smuxi-frontend-gnome",
    "soffice",
    "xfwm4",
]
